import * as _wundergraph_sdk_client from '@wundergraph/sdk/client';
import { OperationsDefinition, OperationRequestOptions, WithInput, ClientResponse, ResponseError, SubscriptionRequestOptions, FetchUserRequestOptions, UploadRequestOptions, ExtractProfileName, ExtractMeta, Client, LogoutOptions } from '@wundergraph/sdk/client';
import { UseQueryOptions as UseQueryOptions$1, UseQueryReturnType, UseMutationReturnType } from '@tanstack/vue-query';
import { Ref } from 'vue';
import { MutationObserverOptions } from '@tanstack/query-core';

type MaybeRef<T> = Ref<T> | T;
type MaybeRefDeep<T> = MaybeRef<T extends Function ? T : T extends object ? {
    [Property in keyof T]: MaybeRefDeep<T[Property]>;
} : T>;

type QueryFetcher<Operations extends OperationsDefinition> = {
    <OperationName extends Extract<keyof Operations['queries'], string>, Data extends Operations['queries'][OperationName]['response'] = Operations['queries'][OperationName]['response'], RequestOptions extends OperationRequestOptions<Extract<keyof Operations['queries'], string>, Operations['queries'][OperationName]['input']> = OperationRequestOptions<Extract<keyof Operations['queries'], string>, Operations['queries'][OperationName]['input']>>(query: RequestOptions): Promise<Data>;
};
type MutationFetcher<Operations extends OperationsDefinition> = {
    <OperationName extends Extract<keyof Operations['mutations'], string>, Data extends Operations['mutations'][OperationName]['response'] = Operations['mutations'][OperationName]['response'], RequestOptions extends OperationRequestOptions<Extract<keyof Operations['mutations'], string>, Operations['mutations'][OperationName]['input']> = OperationRequestOptions<Extract<keyof Operations['mutations'], string>, Operations['mutations'][OperationName]['input']>>(mutation: RequestOptions): Promise<Data>;
};
type QueryKey<Operations extends OperationsDefinition> = {
    <OperationName extends Extract<keyof Operations['queries'], string>, Input extends Operations['queries'][OperationName]['input'] = Operations['queries'][OperationName]['input']>(query: {
        operationName: OperationName;
        input?: Input;
    }): (OperationName | Input | undefined)[];
};
type UseQueryOptions<Data, Error, Input extends object | undefined, OperationName extends string, LiveQuery> = Omit<UseQueryOptions$1<Data, Error, Data, (OperationName | Input | undefined)[]>, 'queryKey' | 'queryFn'> & WithInput<Input, {
    operationName: OperationName;
    liveQuery?: LiveQuery;
    input?: Input;
}>;
type UseQueryHook<Operations extends OperationsDefinition, ExtraOptions extends object = {}> = {
    <OperationName extends Extract<keyof Operations['queries'], string>, Input extends Operations['queries'][OperationName]['input'] = Operations['queries'][OperationName]['input'], Response extends Operations['queries'][OperationName]['response'] = Operations['queries'][OperationName]['response'], LiveQuery extends Operations['queries'][OperationName]['liveQuery'] = Operations['queries'][OperationName]['liveQuery']>(options: UseQueryOptions<Response['data'], Response['error'], Input, OperationName, LiveQuery> & ExtraOptions): UseQueryReturnType<Response['data'], Response['error']> & {
        isSubscribed?: MaybeRef<boolean>;
    };
};
type UseSubscriptionOptions<Data, Error, Input extends object | undefined, OperationName extends string> = WithInput<Input, {
    operationName: OperationName;
    subscribeOnce?: boolean;
    resetOnMount?: boolean;
    enabled?: boolean;
    input?: Input;
    onSuccess?(response: ClientResponse<Data>): void;
    onError?(error: Error): void;
}>;
type UseSubscriptionHook<Operations extends OperationsDefinition, ExtraOptions extends object = {}> = {
    <OperationName extends Extract<keyof Operations['subscriptions'], string>, Input extends Operations['subscriptions'][OperationName]['input'] = Operations['subscriptions'][OperationName]['input'], Response extends Operations['subscriptions'][OperationName]['response'] = Operations['subscriptions'][OperationName]['response']>(options: UseSubscriptionOptions<Response['data'] | undefined, Response['error'], Input, OperationName> & ExtraOptions): UseSubscriptionResult<Response['data'], Response['error']>;
};
type UseSubscriptionResult<Data, Error = ResponseError> = UseQueryReturnType<Data, Error> & {
    isSubscribed: boolean;
};
type UseMutationOptions<Data, Error, Input, OperationName extends string> = Omit<MutationObserverOptions<Data, Error, Input, (OperationName | Input | undefined)[]>, 'mutationKey' | 'mutationFn'> & {
    operationName: OperationName;
};
type UseMutationHook<Operations extends OperationsDefinition, ExtraOptions extends object = {}> = {
    <OperationName extends Extract<keyof Operations['mutations'], string>, Input extends Operations['mutations'][OperationName]['input'] = Operations['mutations'][OperationName]['input'], Response extends Operations['mutations'][OperationName]['response'] = Operations['mutations'][OperationName]['response']>(options: UseMutationOptions<Response['data'], Response['error'], Input, OperationName> & ExtraOptions): UseMutationReturnType<Response['data'], Response['error'], Input, OperationName>;
};
interface RawUseSubscribeToProps extends SubscriptionRequestOptions {
    queryHash: string;
    enabled?: boolean;
    resetOnMount?: boolean;
    onSuccess?(response: ClientResponse): void;
    onError?(error: ResponseError): void;
}
type UseSubscribeToProps = MaybeRef<RawUseSubscribeToProps>;
interface SubscribeToOptions extends SubscriptionRequestOptions {
    onResult(response: ClientResponse): void;
    onSuccess?(response: ClientResponse): void;
    onError?(error: ResponseError): void;
    onAbort?(): void;
}
interface UseUserOptions<User> extends FetchUserRequestOptions, UseQueryOptions$1<User, ResponseError, User, [string]> {
    enabled?: boolean;
}
type UseUserHook<Operations extends OperationsDefinition> = {
    (options?: UseUserOptions<Operations['user']>): UseQueryReturnType<Operations['user'], ResponseError>;
};
type UseUploadOptions = Omit<MutationObserverOptions<string[], ResponseError, UploadRequestOptions, 'uploadFiles'>, 'fetcher'>;
type UseUploadHook<Operations extends OperationsDefinition> = {
    (options?: UseUploadOptions): Omit<MutationObserverOptions<string[], ResponseError, UploadRequestOptions, 'uploadFiles'>, 'mutate'> & {
        upload: <ProviderName extends Extract<keyof Operations['s3Provider'], string>, ProfileName extends ExtractProfileName<Operations['s3Provider'][ProviderName]['profiles']> = ExtractProfileName<Operations['s3Provider'][ProviderName]['profiles']>, Meta extends ExtractMeta<Operations['s3Provider'][ProviderName]['profiles'], ProfileName> = ExtractMeta<Operations['s3Provider'][ProviderName]['profiles'], ProfileName>>(options: UploadRequestOptions<ProviderName, ProfileName, Meta>, config?: UseUploadOptions) => Promise<string[]>;
        uploadAsync: <ProviderName extends Extract<keyof Operations['s3Provider'], string>, ProfileName extends ExtractProfileName<Operations['s3Provider'][ProviderName]['profiles']> = ExtractProfileName<Operations['s3Provider'][ProviderName]['profiles']>, Meta extends ExtractMeta<Operations['s3Provider'][ProviderName]['profiles'], ProfileName> = ExtractMeta<Operations['s3Provider'][ProviderName]['profiles'], ProfileName>>(options: UploadRequestOptions<ProviderName, ProfileName, Meta>, config?: UseUploadOptions) => Promise<string[]>;
    };
};

declare const createHooks: <Operations extends OperationsDefinition<_wundergraph_sdk_client.OperationDefinition, _wundergraph_sdk_client.OperationDefinition, _wundergraph_sdk_client.OperationDefinition, string, _wundergraph_sdk_client.S3ProviderDefinition, string>>(client: Client) => {
    useAuth: () => {
        login: (authProviderID: Operations["authProvider"], redirectURI?: string | undefined) => void;
        logout: (options?: LogoutOptions | undefined) => Promise<boolean>;
    };
    useFileUpload: UseUploadHook<Operations>;
    useUser: UseUserHook<Operations>;
    useMutation: UseMutationHook<Operations, {}>;
    useQuery: UseQueryHook<Operations, {}>;
    useSubscription: UseSubscriptionHook<Operations, {}>;
    queryKey: QueryKey<Operations>;
};

export { MaybeRef, MaybeRefDeep, MutationFetcher, QueryFetcher, QueryKey, RawUseSubscribeToProps, SubscribeToOptions, UseMutationHook, UseMutationOptions, UseQueryHook, UseQueryOptions, UseSubscribeToProps, UseSubscriptionHook, UseSubscriptionOptions, UseSubscriptionResult, UseUploadHook, UseUploadOptions, UseUserHook, UseUserOptions, createHooks };
